#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

import os

from ara.config.base import BaseConfig
from ara.setup import path as ara_location

from ara.tests.unit.common import TestAra


class TestConfig(TestAra):
    """ Tests the config module """
    def setUp(self):
        super(TestConfig, self).setUp()

    def tearDown(self):
        super(TestConfig, self).tearDown()

    # TODO: Improve those
    def test_config_base(self):
        base_config = BaseConfig()
        db = 'sqlite:///%s/ansible.sqlite' % os.path.expanduser('~/.ara')
        defaults = {
            'FREEZER_IGNORE_MIMETYPE_WARNINGS': True,
            'FREEZER_DEFAULT_MIMETYPE': 'text/html',
            'FREEZER_IGNORE_404_NOT_FOUND': True,
            'ARA_DIR': os.path.expanduser('~/.ara'),
            'SQLALCHEMY_DATABASE_URI': db,
            'APPLICATION_ROOT': '/',
            'ARA_HOST': '127.0.0.1',
            'ARA_AUTOCREATE_DATABASE': True,
            'ARA_PORT': "9191",
            'ARA_DATABASE': db,
            'ARA_IGNORE_EMPTY_GENERATION': True,
            'ARA_IGNORE_FACTS': [
                'ansible_env'
            ],
            'ARA_IGNORE_PARAMETERS': [
                'extra_vars'
            ],
            'FREEZER_RELATIVE_URLS': True,
            'SQLALCHEMY_TRACK_MODIFICATIONS': False,
            'SQLALCHEMY_ECHO': False,
            'SQLALCHEMY_POOL_SIZE': None,
            'SQLALCHEMY_POOL_TIMEOUT': None,
            'SQLALCHEMY_POOL_RECYCLE': None,
            'DB_MIGRATIONS': os.path.join(ara_location, 'db')
        }

        for key, value in base_config.config.items():
            assert value == defaults[key]



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

import os
from ara.config.compat import ara_config
from ara.setup import path as ara_location


class BaseConfig(object):
    def __init__(self):
        self.ARA_DIR = ara_config(
            'dir',
            'ARA_DIR',
            os.path.expanduser('~/.ara')
        )
        database_path = os.path.join(self.ARA_DIR, 'ansible.sqlite')
        self.ARA_DATABASE = ara_config(
            'database',
            'ARA_DATABASE',
            'sqlite:///%s' % database_path
        )
        self.ARA_AUTOCREATE_DATABASE = ara_config(
            'autocreate_database',
            'ARA_AUTOCREATE_DATABASE',
            True,
            value_type='boolean'
        )
        self.APPLICATION_ROOT = ara_config(
            'application_root',
            'ARA_APPLICATION_ROOT',
            '/'
        )
        self.SQLALCHEMY_DATABASE_URI = self.ARA_DATABASE
        self.SQLALCHEMY_TRACK_MODIFICATIONS = False
        self.SQLALCHEMY_ECHO = ara_config(
            'sqlalchemy_echo',
            'SQLALCHEMY_ECHO',
            False,
            value_type='boolean'
        )
        self.SQLALCHEMY_POOL_SIZE = ara_config(
            'sqlalchemy_pool_size',
            'SQLALCHEMY_POOL_SIZE',
            None,
            value_type='integer'
        )
        self.SQLALCHEMY_POOL_TIMEOUT = ara_config(
            'sqlalchemy_pool_timeout',
            'SQLALCHEMY_POOL_TIMEOUT',
            None,
            value_type='integer'
        )
        self.SQLALCHEMY_POOL_RECYCLE = ara_config(
            'sqlalchemy_pool_recycle',
            'SQLALCHEMY_POOL_RECYCLE',
            None,
            value_type='integer'
        )
        self.DB_MIGRATIONS = os.path.join(ara_location, 'db')

        self.ARA_HOST = ara_config('host', 'ARA_HOST', '127.0.0.1')
        self.ARA_PORT = ara_config('port', 'ARA_PORT', '9191')
        self.ARA_IGNORE_PARAMETERS = ara_config(
            'ignore_parameters',
            'ARA_IGNORE_PARAMETERS',
            ['extra_vars'],
            value_type='list'
        )
        self.ARA_IGNORE_FACTS = ara_config(
            'ignore_facts',
            'ARA_IGNORE_FACTS',
            ['ansible_env'],
            value_type='list'
        )

        # Static generation with flask-frozen
        self.ARA_IGNORE_EMPTY_GENERATION = ara_config(
            'ignore_empty_generation',
            'ARA_IGNORE_EMPTY_GENERATION',
            True,
            value_type='boolean'
        )
        self.FREEZER_DEFAULT_MIMETYPE = 'text/html'
        self.FREEZER_IGNORE_MIMETYPE_WARNINGS = ara_config(
            'ignore_mimetype_warnings',
            'ARA_IGNORE_MIMETYPE_WARNINGS',
            True,
            value_type='boolean'
        )
        self.FREEZER_RELATIVE_URLS = True
        self.FREEZER_IGNORE_404_NOT_FOUND = True

    @property
    def config(self):
        """ Returns a dictionary for the loaded configuration """
        return {
            key: self.__dict__[key]
            for key in dir(self)
            if key.isupper()
        }



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

from flask import Blueprint
from flask import current_app
from flask import render_template

from ara import models
from ara.utils import fast_count

about = Blueprint('about', __name__)


@about.route('/')
def main():
    """ Returns the about page """
    files = models.File.query
    hosts = models.Host.query
    facts = models.HostFacts.query
    playbooks = models.Playbook.query
    records = models.Data.query
    tasks = models.Task.query
    results = models.TaskResult.query

    if current_app.config['ARA_PLAYBOOK_OVERRIDE'] is not None:
        override = current_app.config['ARA_PLAYBOOK_OVERRIDE']
        files = files.filter(models.File.playbook_id.in_(override))
        facts = (facts
                 .join(models.Host)
                 .filter(models.Host.playbook_id.in_(override)))
        hosts = hosts.filter(models.Host.playbook_id.in_(override))
        playbooks = playbooks.filter(models.Playbook.id.in_(override))
        records = records.filter(models.Data.playbook_id.in_(override))
        tasks = tasks.filter(models.Task.playbook_id.in_(override))
        results = (results
                   .join(models.Task)
                   .filter(models.Task.playbook_id.in_(override)))

    return render_template(
        'about.html',
        active='about',
        files=fast_count(files),
        hosts=fast_count(hosts),
        facts=fast_count(facts),
        playbooks=fast_count(playbooks),
        records=fast_count(records),
        tasks=fast_count(tasks),
        results=fast_count(results)
    )



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

# Note: This file tries to import itself if it's named logging, thus, logger.

import logging
import logging.config
import os
import yaml
from ara.config.compat import ara_config


DEFAULT_LOG_CONFIG = """
---
version: 1
disable_existing_loggers: false
formatters:
    ara_standard:
        format: '%(asctime)s %(levelname)s %(name)s: %(message)s'
handlers:
    ara_console:
        class: logging.StreamHandler
        formatter: ara_standard
        level: INFO
        stream: ext://sys.stdout
    ara_file:
        class: logging.handlers.TimedRotatingFileHandler
        formatter: ara_standard
        level: {level}
        filename: '{dir}/{file}'
        when: 'midnight'
        interval: 1
        backupCount: 30
loggers:
    ara:
        handlers:
            - ara_file
        level: {level}
        propagate: 0
    alembic:
        handlers:
            - ara_console
            - ara_file
        level: WARN
        propagate: 0
    sqlalchemy.engine:
        handlers:
            - ara_file
        level: WARN
        propagate: 0
    werkzeug:
        handlers:
            - ara_console
            - ara_file
        level: INFO
        propagate: 0
"""


class LogConfig(object):
    def __init__(self):
        default_dir = ara_config('dir', 'ARA_DIR',
                                 os.path.expanduser('~/.ara'))
        self.ARA_LOG_CONFIG = ara_config(
            'logconfig', 'ARA_LOG_CONFIG', os.path.join(default_dir,
                                                        'logging.yml')
        )
        self.ARA_LOG_DIR = ara_config('logdir', 'ARA_LOG_DIR', default_dir)
        self.ARA_LOG_FILE = ara_config('logfile', 'ARA_LOG_FILE', 'ara.log')
        self.ARA_LOG_LEVEL = ara_config('loglevel', 'ARA_LOG_LEVEL', 'INFO')
        if self.ARA_LOG_LEVEL == 'DEBUG':
            self.ARA_ENABLE_DEBUG_VIEW = True
        else:
            self.ARA_ENABLE_DEBUG_VIEW = False

    @property
    def config(self):
        """ Returns a dictionary for the loaded configuration """
        return {
            key: self.__dict__[key]
            for key in dir(self)
            if key.isupper()
        }


def setup_logging(config=None):
    if config is None:
        config = LogConfig().config

    if not os.path.isdir(config['ARA_LOG_DIR']):
        os.makedirs(config['ARA_LOG_DIR'], mode=0o750)

    if not os.path.exists(config['ARA_LOG_CONFIG']):
        if not config.get('ARA_LOG_FILE'):
            config['ARA_LOG_FILE'] = 'ara.log'
        default_config = DEFAULT_LOG_CONFIG.format(
            dir=config['ARA_LOG_DIR'],
            file=config['ARA_LOG_FILE'],
            level=config['ARA_LOG_LEVEL']
        )
        with open(config['ARA_LOG_CONFIG'], 'w') as log_config:
            log_config.write(default_config.lstrip())

    ext = os.path.splitext(config['ARA_LOG_CONFIG'])[1]
    if ext in ('.yml', '.yaml', '.json'):
        # yaml.safe_load can load json as well as yaml
        logging.config.dictConfig(
            yaml.safe_load(open(config['ARA_LOG_CONFIG'], 'r'))
        )
    else:
        logging.config.fileConfig(
            config['ARA_LOG_CONFIG'],
            disable_existing_loggers=False
        )

    logger = logging.getLogger('ara.config.logging')
    msg = 'Logging: Level {level} from {config}, logging to {dir}/{file}'
    msg = msg.format(
        level=config['ARA_LOG_LEVEL'],
        config=config['ARA_LOG_CONFIG'],
        dir=config['ARA_LOG_DIR'],
        file=config['ARA_LOG_FILE'],
    )
    logger.debug(msg)



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

# WSGI file to run the ARA server.
# It is expected that the server at least passes an ANSIBLE_CONFIG environment
# variable in order to configure Ansible and ARA.

# Can be configured using environment variables (i.e, Apache SetEnv) with the
# following variables:

# ARA_WSGI_USE_VIRTUALENV
#   Enable virtual environment usage if ARA is installed in a virtual
#   environment.
#   Defaults to '0', set to '1' to enable.
# ARA_WSGI_VIRTUALENV_PATH
#   When using a virtual environment, where the virtualenv is located.
#   Defaults to None, set to the absolute path of your virtualenv.

import os
import logging
import six
import threading

app = None
app_making_lock = threading.Lock()
log = logging.getLogger(__name__)


def application(environ, start_response):
    global app

    # Load virtualenv if necessary
    if (int(environ.get('ARA_WSGI_USE_VIRTUALENV', 0)) == 1 and
       environ.get('ARA_WSGI_VIRTUALENV_PATH')):
        # Backwards compatibility, we did not always suffix activate_this.py
        activate_this = environ.get('ARA_WSGI_VIRTUALENV_PATH')
        if 'activate_this.py' not in activate_this:
            activate_this = os.path.join(activate_this, 'bin/activate_this.py')
        if six.PY2:
            execfile(activate_this, dict(__file__=activate_this))  # nosec
        else:
            exec(open(activate_this).read())  # nosec

    if 'ANSIBLE_CONFIG' in environ:
        os.environ['ANSIBLE_CONFIG'] = environ['ANSIBLE_CONFIG']
    else:
        if 'ANSIBLE_CONFIG' not in os.environ:
            log.warning('ANSIBLE_CONFIG environment variable not found.')

    from ara.webapp import create_app  # flake8: noqa
    from flask import current_app  # flake8: noqa
    with app_making_lock:
        if app is None:
            app = create_app()
    with app.app_context():
        return current_app(environ, start_response)


def main():
    return application



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

# Note (dmsimard): ARA's configuration, views and API imports are done
# "just-in-time" because we might end up loading them for nothing and it has a
# non-negligible cost.
# It also breaks some assumptions... For example, "ara.config" loads the
# configuration automatically on import which might not be desirable.

import datetime
import warnings
import logging
import logging.config
import os
import six
import sys

from ansible import __version__ as ansible_version
from ara import __release__ as ara_release
from ara.config.base import BaseConfig
from ara.config.logger import LogConfig
from ara.config.logger import setup_logging
from ara.config.webapp import WebAppConfig
from ara import models
from ara.utils import fast_count
from ara.utils import playbook_treeview

from alembic.migration import MigrationContext
from alembic.script import ScriptDirectory
from sqlalchemy.engine.reflection import Inspector

from flask import abort
from flask import Flask
from flask import render_template
from flask import send_from_directory
from jinja2 import Markup
from oslo_serialization import jsonutils
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import YamlLexer
from pygments.lexers import JsonLexer
from pygments.lexers.special import TextLexer

with warnings.catch_warnings():
    warnings.filterwarnings('ignore')
    import flask_migrate


def create_app():
    app = Flask('ara')

    configure_app(app)
    configure_dirs(app)
    configure_logging(app)
    configure_app_root(app)
    configure_blueprints(app)
    configure_static_route(app)
    configure_db(app)
    configure_errorhandlers(app)
    configure_template_filters(app)
    configure_context_processors(app)
    configure_cache(app)

    return app


def configure_app(app):
    app.config.update(BaseConfig().config)
    app.config.update(LogConfig().config)
    app.config.update(WebAppConfig().config)


def configure_dirs(app):
    if not os.path.isdir(app.config['ARA_DIR']):
        os.makedirs(app.config['ARA_DIR'], mode=0o700)


def configure_logging(app):
    setup_logging(app.config)


def configure_app_root(app):
    log = logging.getLogger('ara.webapp.configure_app_root')
    # Don't load the middleware needlessly if the root is actually '/'
    if app.config['APPLICATION_ROOT'] != '/':
        app.wsgi_app = AppRootMiddleware(app, app.wsgi_app)
    log.debug('Application root loaded: %s' % app.config['APPLICATION_ROOT'])


def configure_errorhandlers(app):
    @app.errorhandler(404)
    def page_not_found(error):
        return render_template('errors/404.html', error=error), 404


def configure_template_filters(app):
    log = logging.getLogger('ara.filters')

    @app.template_filter('datefmt')
    def jinja_date_formatter(timestamp, format='%Y-%m-%d %H:%M:%S'):
        """ Reformats a datetime timestamp from str(datetime.datetime) """
        if timestamp is None:
            return 'n/a'
        else:
            return datetime.datetime.strftime(timestamp, format)

    @app.template_filter('timefmt')
    def jinja_time_formatter(timestamp):
        """ Reformats a datetime timedelta """
        if timestamp is None:
            return 'n/a'
        else:
            date = datetime.timedelta(seconds=int(timestamp.total_seconds()))
            return str(date)

    @app.template_filter('to_nice_json')
    def jinja_to_nice_json(result):
        """ Tries to format a result as a pretty printed JSON. """
        try:
            return jsonutils.dumps(jsonutils.loads(result),
                                   indent=4,
                                   sort_keys=True)
        except (ValueError, TypeError):
            try:
                return jsonutils.dumps(result, indent=4, sort_keys=True)
            except TypeError as err:
                log.error('failed to dump json: %s', err)
                return result

    @app.template_filter('from_json')
    def jinja_from_json(val):
        try:
            return jsonutils.loads(val)
        except ValueError as err:
            log.error('failed to load json: %s', err)
            return val

    @app.template_filter('yamlhighlight')
    def jinja_yamlhighlight(code):
        formatter = HtmlFormatter(linenos='table',
                                  anchorlinenos=True,
                                  lineanchors='line',
                                  linespans='line',
                                  cssclass='codehilite')

        if not code:
            code = ''

        return highlight(Markup(code).unescape(),
                         YamlLexer(stripall=True),
                         formatter)

    @app.template_filter('pygments_formatter')
    def jinja_pygments_formatter(data):
        formatter = HtmlFormatter(cssclass='codehilite')

        if isinstance(data, dict) or isinstance(data, list):
            data = jsonutils.dumps(data, indent=4, sort_keys=True)
            lexer = JsonLexer()
        elif six.string_types or six.text_type:
            try:
                data = jsonutils.dumps(jsonutils.loads(data),
                                       indent=4,
                                       sort_keys=True)
                lexer = JsonLexer()
            except (ValueError, TypeError):
                lexer = TextLexer()
        else:
            lexer = TextLexer()

        lexer.stripall = True
        return highlight(Markup(data).unescape(), lexer, formatter)

    # TODO: Remove fast_count, replace by "select id, order desc, limit 1"
    @app.template_filter('fast_count')
    def jinja_fast_count(query):
        return fast_count(query)

    @app.template_filter('basename')
    def jinja_basename(pathname):
        return os.path.basename(pathname)

    @app.template_filter('treeview')
    def jinja_treeview(playbook):
        return playbook_treeview(playbook)


def configure_context_processors(app):
    log = logging.getLogger('ara.webapp.context_processors')
    log.debug('Loading context_processors...')

    @app.context_processor
    def ctx_add_nav_data():
        """
        Returns standard data that will be available in every template view.
        """
        try:
            models.Playbook.query.one()
            empty_database = False
        except models.MultipleResultsFound:
            empty_database = False
        except models.NoResultFound:
            empty_database = True

        # Get python version info
        major, minor, micro, release, serial = sys.version_info

        return dict(ara_version=ara_release,
                    ansible_version=ansible_version,
                    python_version="{0}.{1}".format(major, minor),
                    empty_database=empty_database)


def configure_blueprints(app):
    log = logging.getLogger('ara.webapp.configure_blueprints')
    log.debug('Loading blueprints...')

    import ara.views  # flake8: noqa
    views = (
        (ara.views.about, '/about'),
        (ara.views.file, '/file'),
        (ara.views.host, '/host'),
        (ara.views.reports, ''),
        (ara.views.result, '/result'),
    )

    for view, prefix in views:
        app.register_blueprint(view, url_prefix=prefix)

    if app.config.get('ARA_LOG_LEVEL') == 'DEBUG':
        app.register_blueprint(ara.views.debug, url_prefix='/debug')


def configure_db(app):
    """
    0.10 is the first version of ARA that ships with a stable database schema.
    We can identify a database that originates from before this by checking if
    there is an alembic revision available.
    If there is no alembic revision available, assume we are running the first
    revision which contains the latest state of the database prior to this.
    """
    models.db.init_app(app)
    log = logging.getLogger('ara.webapp.configure_db')
    log.debug('Setting up database...')

    if app.config.get('ARA_AUTOCREATE_DATABASE'):
        with app.app_context():
            migrations = app.config['DB_MIGRATIONS']
            flask_migrate.Migrate(app, models.db, directory=migrations)
            config = app.extensions['migrate'].migrate.get_config(migrations)

            # Verify if the database tables have been created at all
            inspector = Inspector.from_engine(models.db.engine)
            if len(inspector.get_table_names()) == 0:
                log.info('Initializing new DB from scratch')
                flask_migrate.upgrade(directory=migrations)

            # Get current alembic head revision
            script = ScriptDirectory.from_config(config)
            head = script.get_current_head()

            # Get current revision, if available
            connection = models.db.engine.connect()
            context = MigrationContext.configure(connection)
            current = context.get_current_revision()

            if not current:
                log.info('Unstable DB schema, stamping original revision')
                flask_migrate.stamp(directory=migrations,
                                    revision='da9459a1f71c')

            if head != current:
                log.info('DB schema out of date, upgrading')
                flask_migrate.upgrade(directory=migrations)


def configure_static_route(app):
    # Note (dmsimard)
    # /static/ is provided from in-tree bundled files and libraries.
    # /static/packaged/ is routed to serve packaged (i.e, XStatic) libraries.
    #
    # The reason why this isn't defined as a proper view by itself is due to
    # a limitation in flask-frozen. Blueprint'd views methods are like so:
    # "<view>.<method>. The URL generator of flask-frozen is a method decorator
    # that expects the method name as the function and, obviously, you can't
    # really have dots in functions.
    # By having the route configured at the root of the application, there's no
    # dots and we can decorate "serve_static_packaged" instead of, say,
    # "static.serve_packaged".

    log = logging.getLogger('ara.webapp.configure_static_route')
    log.debug('Loading static routes...')

    @app.route('/static/packaged/<module>/<path:filename>')
    def serve_static_packaged(module, filename):
        xstatic = app.config['XSTATIC']

        if module in xstatic:
            return send_from_directory(xstatic[module], filename)
        else:
            abort(404)


def configure_cache(app):
    """ Sets up an attribute to cache data in the app context """
    log = logging.getLogger('ara.webapp.configure_cache')
    log.debug('Configuring cache')

    if not getattr(app, '_cache', None):
        app._cache = {}


class AppRootMiddleware(object):
    """
    Middleware to manage route prefixes, for example when hosting ARA in a
    subdirectory.
    """
    def __init__(self, app, wsgi_app):
        self.log = logging.getLogger('ara.webapp.AppRootMiddleware')
        self.log.debug('Initializing AppRootMiddleware')
        self.app = app
        self.wsgi_app = wsgi_app

    def __call__(self, environ, start_response):
        root = self.app.config['APPLICATION_ROOT']
        if environ['PATH_INFO'].startswith(root):
            environ['PATH_INFO'] = environ['PATH_INFO'][len(root):]
            environ['SCRIPT_NAME'] = root
            self.log.debug('Returning with root %s' % root)
            return self.wsgi_app(environ, start_response)
        else:
            self.log.debug('Returning 404 for %s' % environ['PATH_INFO'])
            url = "{scheme}://{host}{root}".format(
                scheme=environ['wsgi.url_scheme'],
                host=environ['HTTP_HOST'],
                root=root,
            )
            msg = """
            This URL doesn't belong to an ARA application. <br />
            <br />
            Did you mean to browse to <a href='{url}'>{url}</a> instead ?
            """.format(url=url)
            start_response('404', [('Content-Type', 'text/html')])
            return [msg.strip().encode()]



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

from xstatic import main as xs
import xstatic.pkg.bootstrap_scss
import xstatic.pkg.datatables
import xstatic.pkg.jquery
import xstatic.pkg.patternfly
import xstatic.pkg.patternfly_bootstrap_treeview
from ara.config.compat import ara_config


class WebAppConfig(object):
    def __init__(self):
        self.ARA_PLAYBOOK_PER_PAGE = ara_config(
            'playbook_per_page',
            'ARA_PLAYBOOK_PER_PAGE',
            10,
            value_type='integer'
        )
        self.ARA_RESULT_PER_PAGE = ara_config(
            'result_per_page',
            'ARA_RESULT_PER_PAGE',
            25,
            value_type='integer'
        )
        self.ARA_PLAYBOOK_OVERRIDE = ara_config(
            'playbook_override',
            'ARA_PLAYBOOK_OVERRIDE',
            None,
            value_type='list'
        )

        treeview = xstatic.pkg.patternfly_bootstrap_treeview
        self.XSTATIC = dict(
            bootstrap=xs.XStatic(xstatic.pkg.bootstrap_scss).base_dir,
            datatables=xs.XStatic(xstatic.pkg.datatables).base_dir,
            jquery=xs.XStatic(xstatic.pkg.jquery).base_dir,
            patternfly=xs.XStatic(xstatic.pkg.patternfly).base_dir,
            patternfly_bootstrap_treeview=xs.XStatic(treeview).base_dir,
        )

    @property
    def config(self):
        """ Returns a dictionary for the loaded configuration """
        return {
            key: self.__dict__[key]
            for key in dir(self)
            if key.isupper()
        }



#  Copyright (c) 2018 Red Hat, Inc.
#
#  This file is part of ARA Records Ansible.
#
#  ARA is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARA is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARA.  If not, see <http://www.gnu.org/licenses/>.

# Compatibility layer between ARA and the different version of Ansible

import warnings
from six.moves import configparser

with warnings.catch_warnings():
    warnings.filterwarnings('ignore')
    import ansible.constants
    from ansible.constants import get_config
    from ansible.config.manager import find_ini_config_file


# Please don't scream deprecated warnings at us
ansible.constants._deprecated = lambda *args: None


def ara_config(key, env_var, default, section='ara', value_type=None):
    """
    Wrapper around Ansible's get_config backward/forward compatibility
    """
    # Bootstrap Ansible configuration
    # Ansible >=2.4 takes care of loading the configuration file itself
    path = find_ini_config_file()
    config = configparser.ConfigParser()
    if path is not None:
        config.read(path)

    return get_config(
        config, section, key, env_var, default, value_type=value_type
    )
